@@ -7,7 +7,7 @@ import tensorflow as tf
 from alibi.api.defaults import DEFAULT_DATA_INTGRAD, DEFAULT_META_INTGRAD
 from alibi.utils.approximation_methods import approximation_parameters
 from alibi.api.interfaces import Explainer, Explanation
-
+from tensorflow.keras.models import Model
 from typing import Callable, TYPE_CHECKING, Union
 
 if TYPE_CHECKING:  # pragma: no cover
@@ -380,11 +380,6 @@ class IntegratedGradients(Explainer):
             for each feature.
 
         """
-        if not tf.executing_eagerly():
-            raise RuntimeError("""To run IntegratedGradients Tensorflow must be executed eagerly.
-            To enable eager execution, add the following lines at the beginning of your script:
-            `import tensorflow as tf`
-            `tf.compat.v1.enable_eager_execution()` """)
 
         if (len(self.model.output_shape) == 1 or self.model.output_shape[1] == 1) and target is None:
             logger.warning("It looks like you are passing a model with a scalar output and target is set to `None`."
@@ -439,6 +434,8 @@ class IntegratedGradients(Explainer):
         # tf concatatation
         grads = tf.concat(batches, 0)
         shape = grads.shape[1:]
+        if isinstance(shape, tf.TensorShape):
+            shape = tuple(shape.as_list())
 
         # invert sign of gradients for target 0 examples if classifier returns only positive class probability
         if (len(self.model.output_shape) == 1 or self.model.output_shape[1] == 1) and target is not None:
@@ -450,7 +447,9 @@ class IntegratedGradients(Explainer):
         # sum integral terms and scale attributions
         sum_int = _sum_integral_terms(step_sizes, grads.numpy())
         if self.layer is not None:
-            norm = (self.layer(X) - self.layer(baselines)).numpy()
+            layer_output = self.layer.output
+            model_layer = Model(self.model.input, outputs=layer_output)
+            norm = (model_layer(X) - model_layer(baselines)).numpy()
         else:
             norm = X - baselines
         attributions = norm * sum_int
